﻿#!/usr/bin/env python
#  -*- coding: utf-8 -*-
#  -*- Python -*-

import os
import sys
import time
import threading
import math

import pygame

#sys.path.append(".")
import RTC
import OpenRTM_aist
from omniORB import CORBA

import JoyVelCtrlWidget

# ////////////////////////////////////////////////////////////////////////////////
## ゲームパッド/ジョイスティックに関する定数
JOYSTICK_ID = 0

# ボタンの名称 (SNES / Wii Classic Controller方式):
ID_BTN_A = 3 #  PSだと○ボタン, XBOXだとBボタン
ID_BTN_B = 2 # 
ID_BTN_X = 1 #
ID_BTN_Y = 0 #
ID_BTN_L = 4 #
ID_BTN_R = 5 #
ID_BTN_START  = 9 # 
ID_BTN_SELECT = 8 # 
BUTTON_PRESSED = 1

AXIS_MAX_FWD_TILT = -1.0 # 前方傾き最大でのaxisの値
AXIS_NEUTRAL      =  0.0 # -0.007843017 # ニュートラル位置でのaxisの値
AXIS_MAX_BWD_TILT =  1.0 # 後方傾き最大でのaxisの値


if False: # ゲームパッド ELECOM JC-U3312SBU
    AXIS_ID_FWD_VEL   = 1 # 左スティックの前後方向の傾き
    AXIS_ID_ANG_VEL   = 2 # 右スティックの左右方向の傾き
    # AXIS_ID_UNDEF0  = 0 # 左スティックの左右方向の傾き
    # AXIS_ID_UNDEF1  = 3 # 右スティックの前後方向の傾き
elif False: # ジョイスティック MadCatz CyborgX
    AXIS_ID_FWD_VEL  = 1 # スティックのpitch    
    AXIS_ID_ANG_VEL  = 0 # スティックのroll
    # AXIS_ID_UNDEF0 = 3 # スティックのyaw
    # AXIS_ID_UNDEF1 = 2 # レバー左側
elif True: # Logicool F310
    AXIS_ID_FWD_VEL  = 1 #     
    AXIS_ID_ANG_VEL  = 4 # 
    # AXIS_ID_UNDEF0 = 3 # 
    # AXIS_ID_UNDEF1 = 2 # 



# ////////////////////////////////////////////////////////////////////////////////
HELP_TEXT = u"""
説明:
  前進/後退速度の操作:    スティック - 前後
  旋回速度の操作:         スティック - 左右
"""


# ゲームパッド, GUI処理用スレッドのループ実行周期
UI_THREAD_PERIOD = 0.1 # [sec]

MAX_FWD_VEL =  0.5 # [m/s]
MIN_FWD_VEL = -0.5 # [m/s]

MAX_ANG_VEL =  0.5*math.pi # [rad/s]
MIN_ANG_VEL = -0.5*math.pi # [rad/s]



#///////////////////////////////////////////////////////////////////////////////
def axisValToFwdVel(ax):
    return ((MAX_FWD_VEL - MIN_FWD_VEL)*
            (ax - AXIS_MAX_BWD_TILT)/(AXIS_MAX_FWD_TILT - AXIS_MAX_BWD_TILT)) + MIN_FWD_VEL

#///////////////////////////////////////////////////////////////////////////////
def axisValToAngVel(ax):
    return ((MAX_ANG_VEL - MIN_ANG_VEL)*
            (ax - AXIS_MAX_BWD_TILT)/(AXIS_MAX_FWD_TILT - AXIS_MAX_BWD_TILT)) + MIN_ANG_VEL



# ////////////////////////////////////////////////////////////////////////////////
_rtc_spec = ["implementation_id", "JoystickVelocityControl", 
             "type_name",         "JoystickVelocityControl", 
             "description",       "Output Velocity2D data for joystick operation", 
             "version",           "0.1", 
             "vendor",            "xxxx", 
             "category",          "Generic", 
             "activity_type",     "DataFlowComponent", 
             "max_instance",      "1", 
             "language",          "Python", 
             "lang_type",         "SCRIPT",
             ""]

# ////////////////////////////////////////////////////////////////////////////////
#!
#
#  コマンド送信コンポーネント
#
class JoystickVelocityControl(OpenRTM_aist.DataFlowComponentBase):

    #!
    #  コンストラクタ
    #      
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)
        
        # print "QuadCommanderGui constructor invoked"

        self.gamepad = None
        self.gui = None
        self.stopUIEvent = threading.Event()

        self.__refVelLock = threading.Lock()
        self.__refVelFwd = 0.0
        self.__refVelAng = 0.0
        
        self.uiThread = None
        self.clockRecs = []
        return


    #!
    #
    # スティックの角度に対応する速度目標値に，UI処理用のスレッドと
    # RTCのスレッドの両方からアクセスするための排他処理を行う
    #  
    def setRefVel(self, fwdVel, angVel):
        # Queue.Queue を使って保存を行うと遅れの要因となるので使わない
        self.__refVelLock.acquire()
        self.__refVelFwd = fwdVel
        self.__refVelAng = angVel
        self.__refVelLock.release()

    #!
    #
    # @see setRefVel()
    #
    # @return タプル (前進/後退速度, 旋回速度)
    def getRefVel(self):
        self.__refVelLock.acquire()
        fwdVel = self.__refVelFwd
        angVel = self.__refVelAng
        self.__refVelLock.release()        
        return fwdVel, angVel

    
    def onInitialize(self):
        self._d_refVel  = RTC.TimedVelocity2D(RTC.Time(0, 0), RTC.Velocity2D(0, 0, 0))
        self._refVelOut = OpenRTM_aist.OutPort("refVel", self._d_refVel)
        self.addOutPort("refVel", self._refVelOut)

        retCode = RTC.RTC_OK
        
        pygame.init()

        if pygame.joystick.get_count() < 1 :
            print "No gamepad/joystick connected"
            retCode = RTC.RTC_ERROR
        else:
            print "Found a gamepad "
            self.gamepad = pygame.joystick.Joystick(JOYSTICK_ID)
            self.gamepad.init()
            print "Initialized the gamepad"

        print "Initialization finished"

        self.gui = JoyVelCtrlWidget.JoyVelCtrlWidget(HELP_TEXT,
                                                     MIN_FWD_VEL, MAX_FWD_VEL,
                                                     MIN_ANG_VEL, MAX_ANG_VEL)
        
        return retCode

    
    
        
    #!
    #  ゲームパッドの処理を担当するスレッドのメインルーチン
    #
    # 
    def uiThreadProc(self):
        while not self.stopUIEvent.is_set():
            # ゲームパッド/ジョイスティック関連のイベント処理
            for event in pygame.event.get():
                if event.type == pygame.JOYAXISMOTION:
                    fwdVel = axisValToFwdVel(self.gamepad.get_axis(AXIS_ID_FWD_VEL))
                    angVel = axisValToAngVel(-self.gamepad.get_axis(AXIS_ID_ANG_VEL))
                    self.setRefVel(fwdVel, angVel)

                elif event.type == pygame.JOYBUTTONDOWN:
                    # ボタンイベントが起きたときにしかget_button()の結果が
                    # 有効でないので判別してから呼び出し
                    if self.gamepad.get_button(ID_BTN_A) == BUTTON_PRESSED:
                        pass
                    
                    if self.gamepad.get_button(ID_BTN_B) == BUTTON_PRESSED:
                        pass
                        
                    if self.gamepad.get_button(ID_BTN_X) == BUTTON_PRESSED:
                        pass
                    
                    if self.gamepad.get_button(ID_BTN_Y) == BUTTON_PRESSED:
                        pass
                        
                    if self.gamepad.get_button(ID_BTN_L) == BUTTON_PRESSED:
                        pass
                    
                    if self.gamepad.get_button(ID_BTN_R) == BUTTON_PRESSED:
                        pass
                        
                    if self.gamepad.get_button(ID_BTN_START) == BUTTON_PRESSED:
                        pass
                        

            fwdVel, angVel = self.getRefVel()
            self.gui.setScaleValues(fwdVel, angVel)
            # print "Scale widget updated"
            time.sleep(UI_THREAD_PERIOD)
        
    
    def onExecute(self, ec_id):
        self._d_refVel.data.vx, self._d_refVel.data.va = self.getRefVel()
        OpenRTM_aist.setTimestamp(self._d_refVel)
        self._refVelOut.write()
        
        return RTC.RTC_OK


    def onActivated(self, ec_id):
        self.gui.start()
        
        self.stopUIEvent.clear()
        self.uiThread = threading.Thread(target=self.uiThreadProc)
        self.uiThread.start()
        return RTC.RTC_OK

    def onDeactivated(self, ec_id):
        self.stopUIEvent.set()
        self.uiThread.join() # self.guiを使う可能性があるのでスレッドの終了を待つ
        
        self.gui.pause()
        del self.uiThread
        
        return RTC.RTC_OK

    def onFinalize(self):
        self.gui.destroy()
        return RTC.RTC_OK

    #def onStartup(self, ec_id):
    #  return RTC.RTC_OK

    #def onShutdown(self, ec_id):
    #  return RTC.RTC_OK
    

    #def onAborting(self, ec_id):
    #  return RTC.RTC_OK
    
    #def onError(self, ec_id):
    #  return RTC.RTC_OK
    
    #def onReset(self, ec_id):
    #  return RTC.RTC_OK
    
    #def onStateUpdate(self, ec_id):
    #  return RTC.RTC_OK
    
    #def onRateChanged(self, ec_id):
    #  return RTC.RTC_OK


def JoystickVelocityControlInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=_rtc_spec)
    manager.registerFactory(profile,
                            JoystickVelocityControl,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    JoystickVelocityControlInit(manager)
    # Create a component
    comp = manager.createComponent("JoystickVelocityControl")



def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()


  
