﻿// -*- C++ -*-
/*!
 * @file  kinectMotion.cpp
 * @brief kinectMotion component
 * @date $Date$
 *
 * $Id$
 */

#include "kinectMotion.h"

// Module specification
// <rtc-template block="module_spec">
static const char* kinectmotion_spec[] =
  {
    "implementation_id", "kinectMotion",
    "type_name",         "kinectMotion",
    "description",       "kinectMotion component",
    "version",           "1.0.0",
    "vendor",            "grou4",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.kinect", "0",

    // Widget
    "conf.__widget__.kinect", "radio",
    // Constraints
    "conf.__constraints__.kinect", "(0,1)",

    "conf.__type__.kinect", "double",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
kinectMotion::kinectMotion(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Direction2In("Direction", m_Direction2),
    m_moveOut("refVel", m_move)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
kinectMotion::~kinectMotion()
{
}



RTC::ReturnCode_t kinectMotion::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("Direction", m_Direction2In);
  
  // Set OutPort buffer
  addOutPort("refVel", m_moveOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("kinect", m_kinect, "0");
  // </rtc-template>
  

  m_move.data.vx = 0.0;
  m_move.data.va = 0.0;
  m_move.data.vy = 0.0;










  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t kinectMotion::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kinectMotion::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kinectMotion::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t kinectMotion::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t kinectMotion::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t kinectMotion::onExecute(RTC::UniqueId ec_id)
{
	if (m_stickIn.isNew()) {
		m_stickIn.read();
	}
	if (m_Direction2In.isNew()) {
		m_Direction2In.read();
	}
	if (m_LRF2In.isNew()) {
		m_LRF2In.read();
	}

	// LRFが障害物を検知したら即停止
	if (m_LRF2.data.va == 0.0 && m_LRF2.data.vx == 0.0 && m_LRF2.data.vy ==0.0) {
		//if (m_LRF2.data.va > 0.01 || m_LRF2.data.va < -0.01 || m_LRF2.data.vx >0.01 || m_LRF2.data.vx < -0.01 || m_LRF2.data.vy >0.01 || m_LRF2.data.vy < -0.01) {
		m_move.data.va = 0;
		m_move.data.vy = 0;
		m_move.data.vx = 0;
		m_moveOut.write();
		return RTC::RTC_OK;
	}

	// LRFは0.0以外の値を出力
	// ジョイスティックの入力を優先
	if (m_stick.data.va > 0.05 || m_stick.data.va < -0.05 || m_stick.data.vx >0.05 || m_stick.data.vx < -0.05 || m_stick.data.vy >0.05 || m_stick.data.vy < -0.05) {
		m_move.data.va = m_stick.data.va / 4;
		m_move.data.vy = m_stick.data.vy / 4;
		m_move.data.vx = m_stick.data.vx / 4;
	} else if (m_Direction2.data == -1.0 || m_Direction2.data == 1.0 || m_Direction2.data == 0.0) {
		// Kinectの値を出力
		m_move.data.vx = 0.08;
		m_move.data.va = 0.0;
		m_move.data.vy = 0.0;
	} else {
		// いずれでもなければ停止
		m_move.data.vx = 0.0;
		m_move.data.va = 0.0;
		m_move.data.vy = 0.0;
	}

	m_moveOut.write();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t kinectMotion::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kinectMotion::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t kinectMotion::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t kinectMotion::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kinectMotion::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void kinectMotionInit(RTC::Manager* manager)
  {
    coil::Properties profile(kinectmotion_spec);
    manager->registerFactory(profile,
                             RTC::Create<kinectMotion>,
                             RTC::Delete<kinectMotion>);
  }
  
};


